import type { Actions, AddEthereumChainParameter } from '@web3-react/types';
import { Connector } from '@web3-react/types';
import type { WalletLink as WalletLinkInstance } from 'walletlink';
import type { WalletLinkOptions } from 'walletlink/dist/WalletLink';
export declare class WalletLink extends Connector {
    /** {@inheritdoc Connector.provider} */
    provider: ReturnType<WalletLinkInstance['makeWeb3Provider']> | undefined;
    private readonly options;
    private eagerConnection?;
    /**
     * A `walletlink` instance.
     */
    walletLink: WalletLinkInstance | undefined;
    /**
     * @param options - Options to pass to `walletlink`
     * @param connectEagerly - A flag indicating whether connection should be initiated when the class is constructed.
     */
    constructor(actions: Actions, options: WalletLinkOptions & {
        url: string;
    }, connectEagerly?: boolean);
    private connectListener;
    private disconnectListener;
    private chainChangedListener;
    private accountsChangedListener;
    private initialize;
    /**
     * Initiates a connection.
     *
     * @param desiredChainIdOrChainParameters - If defined, indicates the desired chain to connect to. If the user is
     * already connected to this chain, no additional steps will be taken. Otherwise, the user will be prompted to switch
     * to the chain, if one of two conditions is met: either they already have it added, or the argument is of type
     * AddEthereumChainParameter, in which case the user will be prompted to add the chain with the specified parameters
     * first, before being prompted to switch.
     */
    activate(desiredChainIdOrChainParameters?: number | AddEthereumChainParameter): Promise<void>;
    /** {@inheritdoc Connector.deactivate} */
    deactivate(): void;
}
